//
//  AboutViewController.m
//  IMSAppDemo
//
//  Created by Dong on 2017/11/16.
//  Copyright © 2017年 Aliyun.com. All rights reserved.
//

#import "AboutViewController.h"
#import <CloudPushSDK/CloudPushSDK.h>
#import <AlinkAppExpress/LKAppExpress.h>
#import <IMSLog/IMSLog.h>
#import "LegalViewController.h"
#import <IMSApiClient/IMSConfiguration.h>
#import "IMSSDKEntryProtocol.h"
#import "UIViewController+HUD.h"

#define COPYRIGHT_TAG 99

@interface AboutViewController () <UITableViewDelegate, UITableViewDataSource>
@property(nonatomic, strong) UITableView *table;
@end

@implementation AboutViewController

+ (instancetype)sharedInstance {
  static AboutViewController *instance;

  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
    instance = [[[self class] alloc] init];
  });

  return instance;
}

- (void)viewDidLoad {
  [super viewDidLoad];
  // Do any additional setup after loading the view.
  self.title = @"关于";
  self.view.backgroundColor = [UIColor colorWithRed:239.0 / 255
                                              green:238.0 / 255
                                               blue:241.0 / 255
                                              alpha:1];

  [self createTable];
  [self createCopyright];
}

- (void)viewWillAppear:(BOOL)animated {
  [super viewWillAppear:animated];
  [self.navigationController setNavigationBarHidden:NO];
}

- (void)createCopyright {
    UILabel *label = [[UILabel alloc] init];
    label.tag = COPYRIGHT_TAG;
    CGFloat y = self.view.frame.size.height - 20;
    label.frame = CGRectMake((self.view.frame.size.width - 260) / 2, y, 260, 17);
    label.text = @"© 2009-2017 Aliyun.com. All rights reserved.";
    label.font = [UIFont fontWithName:@"PingFangSC-Light" size:12];
    label.textAlignment = NSTextAlignmentCenter;
    label.textColor = [UIColor colorWithRed:85 / 255.0
                                      green:85 / 255.0
                                       blue:85 / 255.0
                                      alpha:1 / 1.0];
    [self.view addSubview:label];
}

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    
    UIView *copyRightLabel = [self.view viewWithTag:COPYRIGHT_TAG];
    if (copyRightLabel) {
        CGFloat y = self.view.frame.size.height - 8 - 17;
        if (@available(iOS 11.0, *)) {
            y -= self.view.safeAreaInsets.bottom;
        }
        copyRightLabel.frame = CGRectMake((self.view.frame.size.width - 260) / 2, y, 260, 17);
    }
}

- (void)createTable {
  self.table = [[UITableView alloc] initWithFrame:self.view.bounds
                                            style:UITableViewStylePlain];
  [self.view addSubview:self.table];
  self.table.backgroundColor = [UIColor colorWithRed:239 / 255.0
                                               green:238 / 255.0
                                                blue:241 / 255.0
                                               alpha:1.0];
  self.table.delegate = self;
  self.table.dataSource = self;
  UIView *header = [[UIView alloc]
      initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, 140)];
  self.table.tableHeaderView = header;
  self.table.tableFooterView = [[UIView alloc] init];
  self.table.estimatedRowHeight = 0;
  self.table.estimatedSectionHeaderHeight = 0;
  self.table.estimatedSectionFooterHeight = 0;

  UIImageView *logo = [[UIImageView alloc]
      initWithFrame:CGRectMake(0.5 * (self.view.frame.size.width - 254),
                               0.5 * (140 - 92), 254, 92)];

  logo.image = [UIImage imageNamed:@"iotlogo"];

  [header addSubview:logo];
  self.table.separatorColor = [UIColor colorWithRed:234 / 255.0
                                              green:234 / 255.0
                                               blue:234 / 255.0
                                              alpha:1 / 1.0];
    
}

- (CGFloat)tableView:(UITableView *)tableView
    heightForHeaderInSection:(NSInteger)section {
  if (section == 0) {
    return 0;
  }

  return 20;
}

- (CGFloat)tableView:(UITableView *)tableView
    heightForRowAtIndexPath:(NSIndexPath *)indexPath {
  return 50;
}

- (NSInteger)tableView:(UITableView *)tableView
    numberOfRowsInSection:(NSInteger)section {
  if (section == 0) {
    return 4;
  }

  return 2;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
  return 2;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath {
  static NSString *identifier = @"defaultCell";
  UITableViewCell *cell =
      [tableView dequeueReusableCellWithIdentifier:identifier];
  if (cell == nil) {
    cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1
                                  reuseIdentifier:identifier];
  }

  cell.accessoryType = UITableViewCellAccessoryNone;
  if (indexPath.section == 0) {
    if (indexPath.row == 0) {
      cell.textLabel.text = @"AppKey";
      cell.detailTextLabel.text = [[IMSConfiguration sharedInstance] appKey];
    }
    if (indexPath.row == 1) {
      cell.textLabel.text = @"ClientID";
      NSString *clientId = [[LKAppExpress sharedInstance] getClientId];
      if (clientId == nil || [clientId length] == 0) {
        clientId = @"";
      }
      cell.detailTextLabel.text = clientId;
    }
    if (indexPath.row == 2) {
      cell.textLabel.text = @"DeviceID";
      if ([[CloudPushSDK getDeviceId] length] > 0) {
        cell.detailTextLabel.text = [CloudPushSDK getDeviceId];
      }
    }
    if (indexPath.row == 3) {
      cell.textLabel.text = @"DeviceToken";
      if ([[CloudPushSDK getApnsDeviceToken] length] > 0) {
        cell.detailTextLabel.text = [CloudPushSDK getApnsDeviceToken];
      }
    }
  }

  if (indexPath.section == 1) {
    if (indexPath.row == 0) {
      cell.textLabel.text = @"当前版本";
      NSDictionary *infoDic = [[NSBundle mainBundle] infoDictionary];
      NSString *localVersion =
          [infoDic objectForKey:@"CFBundleShortVersionString"];
      if (localVersion == nil || [localVersion length] == 0) {
        localVersion = @"";
      }
      cell.detailTextLabel.text = localVersion;
    }
    if (indexPath.row == 1) {
      cell.textLabel.text = @"版权声明";
      cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
  }

  cell.detailTextLabel.font = [UIFont fontWithName:@"PingFangSC-Light" size:14];
  cell.detailTextLabel.textColor = [UIColor colorWithRed:85 / 255.0
                                                   green:85 / 255.0
                                                    blue:85 / 255.0
                                                   alpha:1 / 1.0];

  cell.textLabel.font = [UIFont fontWithName:@"PingFangSC-Regular" size:16];
  cell.textLabel.textColor = [UIColor colorWithRed:51 / 255.0
                                             green:51 / 255.0
                                              blue:51 / 255.0
                                             alpha:1 / 1.0];

  return cell;
}

- (void)tableView:(UITableView *)tableView
    didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  [tableView deselectRowAtIndexPath:indexPath animated:YES];
  if (indexPath.row == 1 && indexPath.section == 1) {
    LegalViewController *legal = [[LegalViewController alloc] init];
    [self.navigationController pushViewController:legal animated:YES];
  }
}

- (void)didReceiveMemoryWarning {
  [super didReceiveMemoryWarning];
  // Dispose of any resources that can be recreated.
}

#pragma mark - Copying text

- (BOOL)tableView:(UITableView *)tableView
    shouldShowMenuForRowAtIndexPath:(NSIndexPath *)indexPath {
  return YES;
}

- (BOOL)tableView:(UITableView *)tableView
     canPerformAction:(SEL)action
    forRowAtIndexPath:(NSIndexPath *)indexPath
           withSender:(id)sender {
  return action == @selector(copy:);
}

- (void)tableView:(UITableView *)tableView
        performAction:(SEL)action
    forRowAtIndexPath:(NSIndexPath *)indexPath
           withSender:(id)sender {
  if (action == @selector(copy:)) {
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    NSString *textToCopy = cell.detailTextLabel.text;
    if (textToCopy) {
      UIPasteboard.generalPasteboard.string = textToCopy;
      IMSLogDebug(@"IMSDemo", @"已复制到剪贴板: %@", textToCopy);
        [self ims_showHUDWithMessage:[NSString stringWithFormat:@"已复制到剪贴板: %@", textToCopy]];
    }
  }
}

/*
 #pragma mark - Navigation

 // In a storyboard-based application, you will often want to do a little
 preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

@end
