//
//  AppDelegate.m
//  IMSAppDemo
//
//  Created by Hager Hu on 07/11/2017.
//  Copyright © 2017 Aliyun.com. All rights reserved.
//

#import "AppDelegate.h"
#import <IMSLog/IMSLog.h>
#import <IMSLogDemo/IMSDashboardLogAssistant.h>
#import <CloudPushSDK/CloudPushSDK.h>
#import "AboutViewController.h"
#import <IMSApiClient/IMSConfiguration.h>
#import <objc/runtime.h>
#import "IMSSDKEntryProtocol.h"

#define IMS_DEMO_TAG @"IMSDemo"

@interface SDKEntry ()

@end
@implementation SDKEntry
@end

@interface AppDelegate ()

@end

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application
    didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
  // Override point for customization after application launch.

  [IMSLog setAllTagsLevel:IMSLogLevelVerbose];
  [IMSLog showInConsole:YES];
  IMSDashboardLogAssistant *plug = [[IMSDashboardLogAssistant alloc] init];
  [IMSLog addAssistant:plug];

  [IMSLog registerTag:IMS_DEMO_TAG];

  [self getAllClasses];
  // APNs注册，获取deviceToken并上报
  [self registerAPNS:application];
  // 初始化推送SDK
  [self initCloudPush];
  // 监听推送通道打开动作
  //[self listenerOnChannelOpened];
  // 监听推送消息到达
  [self registerMessageReceive];
  // 点击通知将App从关闭状态启动时，将通知打开回执上报
  // [CloudPushSDK handleLaunching:launchOptions];(Deprecated from v1.8.1)
  [CloudPushSDK sendNotificationAck:launchOptions];

  return YES;
}

// 寻找实现SDK展现模块入口协议的类 并在首页列表展示
- (void)getAllClasses {
  int numClasses;
  Class *classes = NULL;
  self.allSDKEntries = [[NSMutableArray alloc] init];

  classes = NULL;
  numClasses = objc_getClassList(NULL, 0);

  if (numClasses > 0) {
    classes = (__unsafe_unretained Class *)malloc(sizeof(Class) * numClasses);
    numClasses = objc_getClassList(classes, numClasses);
    for (int i = 0; i < numClasses; i++) {
      Class c = classes[i];
      // NSLog(@"%s", class_getName(c));
      NSString *className = [NSString stringWithFormat:@"%s", class_getName(c)];
      if ([className hasPrefix:@"SDKEntry"] &&
          [c conformsToProtocol:@protocol(IMSSDKEntryProtocol)]) {
        SDKEntry *entry = [[SDKEntry alloc] init];
        entry.SDKClassName = className;
        if ([c respondsToSelector:@selector(getSDKDisplayConfig)]) {
          entry.SDKDisplayConfig =
              [c performSelector:@selector(getSDKDisplayConfig)];
          if ([entry.SDKDisplayConfig objectForKey:@"DisplayLevel"]) {
            entry.displayLevel = [
                [entry.SDKDisplayConfig objectForKey:@"DisplayLevel"] intValue];
          } else {
            entry.displayLevel = 99; //默认排在后面
          }
        }

        if ([c respondsToSelector:@selector(getInitLevel)]) {
          entry.initLevel = [c getInitLevel];
        } else {
          entry.initLevel = 9.9; //默认低优先级启动
        }

        [self.allSDKEntries addObject:entry];
      }
    }
    free(classes);
  }

  //依据initLevel顺序初始化
  NSMutableArray *sortArray = [self.allSDKEntries copy];

  NSArray *sortData = [sortArray sortedArrayWithOptions:NSSortStable
                                        usingComparator:^NSComparisonResult(
                                            SDKEntry *obj1, SDKEntry *obj2) {
                                          float value1 = [obj1 initLevel];
                                          float value2 = [obj2 initLevel];
                                          if (value1 > value2) {
                                            return NSOrderedDescending;
                                          } else if (value1 == value2) {
                                            return NSOrderedSame;
                                          } else {
                                            return NSOrderedAscending;
                                          }
                                        }];
  for (SDKEntry *entry in sortData) {
    if ([NSClassFromString(entry.SDKClassName)
            conformsToProtocol:@protocol(IMSSDKEntryProtocol)]) {
      [NSClassFromString(entry.SDKClassName)
          performSelector:@selector(SDKEntryInit)];
    }
  }

  //修改显示顺序

  NSArray *displayArray =
      [sortArray sortedArrayWithOptions:NSSortStable
                        usingComparator:^NSComparisonResult(SDKEntry *obj1,
                                                            SDKEntry *obj2) {
                          float value1 = [obj1 displayLevel];
                          float value2 = [obj2 displayLevel];
                          if (value1 > value2) {
                            return NSOrderedDescending;
                          } else if (value1 == value2) {
                            return NSOrderedSame;
                          } else {
                            return NSOrderedAscending;
                          }
                        }];

  self.allSDKEntries = [[NSMutableArray alloc] initWithArray:displayArray];
}

- (void)initCloudPush {
  // 初始化推送SDK
  [CloudPushSDK setAuthCode:[[IMSConfiguration sharedInstance] authCode]];
  [CloudPushSDK asyncInitWithSecurity:^(CloudPushCallbackResult *res) {
    if (res.success) {
      IMSLogDebug(IMS_DEMO_TAG, @"Push SDK init success, deviceId: %@.",
                  [CloudPushSDK getDeviceId]);
    } else {
      IMSLogDebug(IMS_DEMO_TAG, @"Push SDK init failed, error: %@", res.error);
    }
  }];
}

/*
 *  App处于启动状态时，通知打开回调
 */
- (void)application:(UIApplication *)application
    didReceiveRemoteNotification:(NSDictionary *)userInfo {
  IMSLogDebug(IMS_DEMO_TAG, @"Receive one notification.");
  // 取得APNS通知内容
  NSDictionary *aps = [userInfo valueForKey:@"aps"];
  // 内容
  NSString *content = [aps valueForKey:@"alert"];
  // badge数量
  NSInteger badge = [[aps valueForKey:@"badge"] integerValue];
  // 播放声音
  NSString *sound = [aps valueForKey:@"sound"];
  // 取得Extras字段内容
  NSString *Extras =
      [userInfo valueForKey:@"Extras"]; //服务端中Extras字段，key是自己定义的
  IMSLogDebug(IMS_DEMO_TAG,
              @"content = [%@], badge = [%ld], sound = [%@], Extras = [%@]",
              content, (long)badge, sound, Extras);
  // iOS badge 清0
  application.applicationIconBadgeNumber = 0;
  // 通知打开回执上报
  // [CloudPushSDK handleReceiveRemoteNotification:userInfo];(Deprecated from
  // v1.8.1)
  [CloudPushSDK sendNotificationAck:userInfo];
}

- (void)applicationWillResignActive:(UIApplication *)application {
  // Sent when the application is about to move from active to inactive state.
  // This can occur for certain types of temporary interruptions (such as an
  // incoming phone call or SMS message) or when the user quits the application
  // and it begins the transition to the background state.
  // Use this method to pause ongoing tasks, disable timers, and invalidate
  // graphics rendering callbacks. Games should use this method to pause the
  // game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
  // Use this method to release shared resources, save user data, invalidate
  // timers, and store enough application state information to restore your
  // application to its current state in case it is terminated later.
  // If your application supports background execution, this method is called
  // instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
  // Called as part of the transition from the background to the active state;
  // here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
  // Restart any tasks that were paused (or not yet started) while the
  // application was inactive. If the application was previously in the
  // background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application {
  // Called when the application is about to terminate. Save data if
  // appropriate. See also applicationDidEnterBackground:.
}

/**
 *    注册苹果推送，获取deviceToken用于推送
 *
 *    @param     application
 */
- (void)registerAPNS:(UIApplication *)application {
    // iOS 8 Notifications
    [application registerUserNotificationSettings:
                     [UIUserNotificationSettings
                         settingsForTypes:(UIUserNotificationTypeSound |
                                           UIUserNotificationTypeAlert |
                                           UIUserNotificationTypeBadge)
                               categories:nil]];
[application registerForRemoteNotifications];
}
/*
 *  苹果推送注册成功回调，将苹果返回的deviceToken上传到CloudPush服务器
 */
- (void)application:(UIApplication *)application
    didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
  [CloudPushSDK
      registerDevice:deviceToken
        withCallback:^(CloudPushCallbackResult *res) {
          if (res.success) {
            IMSLogDebug(IMS_DEMO_TAG, @"Register deviceToken success: %@",
                        [CloudPushSDK getApnsDeviceToken]);
          } else {
            IMSLogDebug(IMS_DEMO_TAG, @"Register deviceToken failed, error: %@",
                        res.error);
          }
        }];
}

/*
 *  苹果推送注册失败回调
 */
- (void)application:(UIApplication *)application
    didFailToRegisterForRemoteNotificationsWithError:(NSError *)error {
  IMSLogDebug(IMS_DEMO_TAG,
              @"didFailToRegisterForRemoteNotificationsWithError %@", error);
}

/**
 *    注册推送消息到来监听
 */
- (void)registerMessageReceive {
  [[NSNotificationCenter defaultCenter]
      addObserver:self
         selector:@selector(onMessageReceived:)
             name:@"CCPDidReceiveMessageNotification"
           object:nil];
}
/**
 *    处理到来推送消息
 *
 *    @param   notification
 */
- (void)onMessageReceived:(NSNotification *)notification {
  CCPSysMessage *message = [notification object];
  NSString *title = [[NSString alloc] initWithData:message.title
                                          encoding:NSUTF8StringEncoding];
  NSString *body = [[NSString alloc] initWithData:message.body
                                         encoding:NSUTF8StringEncoding];
  IMSLogDebug(IMS_DEMO_TAG, @"Receive message title: %@, content: %@.", title,
              body);
}


@end

