//
//  ViewController.m
//  IMSAppDemo
//
//  Created by Hager Hu on 07/11/2017.
//  Copyright © 2017 Aliyun.com. All rights reserved.
//

#import "ViewController.h"
#import <IMSScanner/IMSScanner.h>
#import "AboutViewController.h"
#import <IMSLogDemo/AKDebugDashboard.h>
#import <IMSAccount/IMSAccountService.h>
#import <IMSBoneKitDemo/BoneDemoProxy.h>
#import <IMSLog/IMSLog.h>
#import "AppDelegate.h"
#import "IMSSDKEntryProtocol.h"

@interface ViewController () <UITableViewDataSource, UITableViewDelegate>

@property(nonatomic, strong) NSMutableArray *dataList;
@property(nonatomic, strong) UIImageView *userImage;

@property(nonatomic, strong) IBOutlet UITableView *tableView;

@end

@implementation ViewController

- (void)viewDidLoad {
  [super viewDidLoad];

  [self.view layoutIfNeeded];
  self.tableView.dataSource = self;
  self.tableView.delegate = self;
  [self createAPILevel];
  [self createFloatPanel];
  [self createLogin];
  [self formatUI];
  self.title = @"DemoApp";
  AppDelegate *appDelegate =
      (AppDelegate *)[[UIApplication sharedApplication] delegate];
  self.dataList = appDelegate.allSDKEntries;
  self.tableView.separatorColor=[UIColor colorWithRed:234 / 255.0
                                              green:234 / 255.0
                                               blue:234 / 255.0
                                              alpha:1 / 1.0];
    
}

- (void)didReceiveMemoryWarning {
  [super didReceiveMemoryWarning];
}

- (void)viewWillAppear:(BOOL)animated {
  [self.navigationController setNavigationBarHidden:NO];
  [super viewWillAppear:animated];
  [self updateLoginStatus];
}

#pragma mark - UITableViewDataSource

- (CGFloat)tableView:(UITableView *)tableView
    heightForRowAtIndexPath:(NSIndexPath *)indexPath {

  SDKEntry *entry = self.dataList[indexPath.row];
  NSArray *actions = [entry.SDKDisplayConfig objectForKey:@"Actions"];
  int smallScreen = self.view.bounds.size.width == 320 ? 33 : 0;
  if (indexPath.section == 0 && indexPath.row == 0) {
    return 120 + smallScreen;
  }

  if ([actions count] > 3) {
    return 90 + smallScreen;
  } else {
    return 90;
  }
}

- (NSInteger)tableView:(UITableView *)tableView
    numberOfRowsInSection:(NSInteger)section {
  return [self.dataList count];
}

- (void)setButtonStyle:(UIButton *)button {
  UIColor *styleColor = [UIColor colorWithRed:71 / 255.0
                                        green:193 / 255.0
                                         blue:223 / 255.0
                                        alpha:1 / 1.0];
  button.titleLabel.font = [UIFont fontWithName:@"PingFangSC-Regular" size:12];
  [button setTitleColor:styleColor forState:UIControlStateNormal];
  [button.layer setMasksToBounds:YES];
  [button.layer setCornerRadius:2.0];
  [button.layer setBorderWidth:0.5];
  button.layer.borderColor = styleColor.CGColor;
}

- (void)setDebugButtonStyle:(UIButton *)button {
  UIColor *styleColor = [UIColor colorWithRed:60 / 255.0
                                        green:177 / 255.0
                                         blue:79 / 255.0
                                        alpha:1 / 1.0];
  button.titleLabel.font = [UIFont fontWithName:@"PingFangSC-Regular" size:12];
  [button setTitleColor:styleColor forState:UIControlStateNormal];
  [button.layer setMasksToBounds:YES];
  [button.layer setCornerRadius:2.0];
  [button.layer setBorderWidth:0.5];
  button.layer.borderColor = styleColor.CGColor;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath {
  static NSString *identifier = @"defaultCell";
  UITableViewCell *cell =
      [tableView dequeueReusableCellWithIdentifier:identifier];
  UILabel *label = (UILabel *)[cell viewWithTag:100];
  UILabel *online = (UILabel *)[cell viewWithTag:200];
  if (cell == nil) {
    cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1
                                  reuseIdentifier:identifier];
    label = [[UILabel alloc] init];
    label.frame = CGRectMake(15, 16, 283, 18);
    label.font = [UIFont fontWithName:@"PingFangSC-Regular" size:16];
    label.textColor = [UIColor colorWithRed:51 / 255.0
                                      green:51 / 255.0
                                       blue:51 / 255.0
                                      alpha:1 / 1.0];
    [cell addSubview:label];
    label.tag = 100;

    if (indexPath.row == 0 && indexPath.section == 0) {
      UIView *greenview = [[UIView alloc] init];
      greenview.frame = CGRectMake(15, 44, 8, 8);
      greenview.backgroundColor = [UIColor colorWithRed:53 / 255.0
                                                  green:179 / 255.0
                                                   blue:74 / 255.0
                                                  alpha:1 / 1.0];
      greenview.clipsToBounds = YES;
      greenview.layer.cornerRadius = 4;
      [cell addSubview:greenview];

      online = [[UILabel alloc] init];
      online.frame = CGRectMake(31, 38, 283, 24);
      online.font = [UIFont fontWithName:@"PingFangSC-Regular" size:12];
      online.textColor = [UIColor colorWithRed:102 / 255.0
                                         green:102 / 255.0
                                          blue:102 / 255.0
                                         alpha:1 / 1.0];
      [cell addSubview:online];
      online.tag = 200;
    }

    cell.tag = indexPath.row;
    for (int i = 101; i < 105; i++) {
      UIButton *button = [[UIButton alloc] initWithFrame:CGRectZero];
      button.tag = i;
      [cell addSubview:button];
      [button addTarget:self
                    action:@selector(clicked:)
          forControlEvents:UIControlEventTouchUpInside];
    }
  }

  NSMutableArray *buttons = [[NSMutableArray alloc] init];
  for (int i = 1; i < 5; i++) {
    if ([cell viewWithTag:100 + i]) {
      [buttons addObject:(UIButton *)[cell viewWithTag:100 + i]];
    }
  }

  SDKEntry *entry = self.dataList[indexPath.row];
  entry.SDKDisplayConfig = [NSClassFromString(entry.SDKClassName)
      performSelector:@selector(getSDKDisplayConfig)];
  label.text = [entry.SDKDisplayConfig objectForKey:@"DisplayName"];
  NSArray *actions = [entry.SDKDisplayConfig objectForKey:@"Actions"];
  int j = 0;
  int bianju = 16;

  for (UIButton *button in buttons) {
    [self setButtonStyle:button];
    if (j < [actions count]) {
      int secondLine = 0;
      if (j == 3 && self.view.bounds.size.width == 320) {
        secondLine = 40;
        bianju = 16;
      }
      if (indexPath.row == 0 && indexPath.section == 0) {
        secondLine = secondLine + 25;
        online.text =
            [entry.SDKDisplayConfig objectForKey:@"OnlineStatus"] ?: @"";
      }

      NSString *actionText = [actions objectAtIndex:j];
      [button setTitle:actionText forState:UIControlStateNormal];
      CGSize actionSize =
          [actionText
              boundingRectWithSize:CGSizeMake(300, button.frame.size.height)
                           options:NSStringDrawingUsesLineFragmentOrigin
                        attributes:@{
                          NSFontAttributeName : button.titleLabel.font
                        }
                           context:nil]
              .size;
      int actionWidth = actionSize.width + 36;
      if ([actionText isEqualToString:@"调试"] ||
          [actionText isEqualToString:@"本地调试"]) {
        [self setDebugButtonStyle:button];
      }
      button.frame = CGRectMake(bianju, 45 + secondLine, actionWidth, 28);
      bianju = bianju + actionWidth + 5;
      button.hidden = NO;
    } else {
      button.hidden = YES;
    }
    j++;
  }

  cell.separatorInset = UIEdgeInsetsZero;
  if ([cell respondsToSelector:@selector(setLayoutMargins:)]) {
    [cell setLayoutMargins:UIEdgeInsetsZero];
  }
  if ([cell
          respondsToSelector:@selector(setPreservesSuperviewLayoutMargins:)]) {
    [cell setPreservesSuperviewLayoutMargins:NO];
  }

  return cell;
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UITableView *)tableView
    didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

#pragma mark - Events

- (void)clicked:(UIButton *)button {
  int row = (int)[button superview].tag;
  SDKEntry *entry = self.dataList[row];
  Class selecteSDK = NSClassFromString(entry.SDKClassName);
  if ([selecteSDK conformsToProtocol:@protocol(IMSSDKEntryProtocol)]) {
    int selectedIndex = (int)button.tag - 101;
    [selecteSDK performSelector:@selector(didSelected:from:)
                     withObject:[NSNumber numberWithInt:selectedIndex]
                     withObject:self.navigationController];
  }
}

- (void)dragMoving:(UIButton *)btn
         withEvent:(UIEvent *)event
{
    UITouch *touch = event.allTouches.anyObject;
    
    CGPoint newPt = [touch locationInView:self.view];
    CGPoint prePt = [touch previousLocationInView:self.view];
    
    CGFloat dX = newPt.x - prePt.x;
    CGFloat dY = newPt.y - prePt.y;
    CGFloat newX = btn.center.x + dX;
    CGFloat newY = btn.center.y + dY;
    
    CGFloat left = btn.frame.size.width / 2;
    CGFloat top = btn.frame.size.height / 2;
    CGFloat right = [UIScreen mainScreen].bounds.size.width - btn.frame.size.width / 2;
    CGFloat bottom = [UIScreen mainScreen].bounds.size.height - btn.frame.size.height / 2;
    
    if (@available(iOS 11.0, *)) {
        UIEdgeInsets keyWindowSafeAreaInsets = UIApplication.sharedApplication.keyWindow.safeAreaInsets;
        left += keyWindowSafeAreaInsets.left;
        top += keyWindowSafeAreaInsets.top;
        right -= keyWindowSafeAreaInsets.right;
        bottom -= keyWindowSafeAreaInsets.bottom;
    }
    
    if (newX < left) newX = left;
    if (newX > right) newX = right;
    if (newY < top) newY = top;
    if (newY > bottom) newY = bottom;
    
    btn.center = CGPointMake(newX, newY);
}

- (void)goAbout {
    UIViewController *topVC = self.navigationController.topViewController;
    if (![topVC isKindOfClass:AboutViewController.class]) {
        AboutViewController *about = [[AboutViewController alloc] init];
        [self.navigationController pushViewController:about animated:YES];
    }
}

//扫码
- (void)goScan {
  IMSScanViewController *scanner = [[IMSScanViewController alloc] init];
  [self presentViewController:scanner animated:YES completion:nil];
  __weak ViewController *weakSelf = self;
  __weak IMSScanViewController *weakScanner = scanner;
  scanner.resultBlock = ^(NSString *value) {
    NSURL *url = [NSURL URLWithString:value];
    [weakScanner
        dismissViewControllerAnimated:NO
                           completion:^{
                             if ([url.host
                                     isEqualToString:@"bonemobile.local"] &&
                                 [value rangeOfString:@"boneDebugHost"].length >
                                     0) {
                               BoneDemoProxy *proxy =
                                   [BoneDemoProxy proxyWithController:weakSelf];
                               [proxy handleBoneQRCode:value];
                             } else {
                               UIAlertView *alertView = [[UIAlertView alloc]
                                       initWithTitle:
                                           @"扫"
                                           @"码失败！请检查二维码是"
                                           @"否正确!"
                                             message:nil
                                            delegate:nil
                                   cancelButtonTitle:@"OK"
                                   otherButtonTitles:nil];
                               [alertView show];
                             }
                           }];
  };
}

//日志模块
- (void)goLog {
  [[AKDebugDashboard sharedDashboard] setFullScreen:YES];
}

- (void)loginEvent {
  if ([[IMSAccountService sharedService] isLogin]) {
    [self logOutAction];
  } else {
    [[IMSAccountService sharedService] showLoginWithController:self
        success:^(NSDictionary *userInfo) {
          IMSLogDebug(@"IMSDemo", @"%@", userInfo);
        }
        failure:^(NSError *error) {
          IMSLogDebug(@"IMSDemo", @"%@", error.localizedDescription);
        }];
  }
}

- (void)logOutAction {
  NSString *userName = [[[IMSAccountService sharedService] currentSession]
      objectForKey:ACCOUNT_NICKNAME_KEY];
  if (userName == nil || [userName length] == 0) {
    userName = @"";
  }

  UIAlertController *actionSheetController = [UIAlertController
      alertControllerWithTitle:@"当前登录帐号"
                       message:userName
                preferredStyle:UIAlertControllerStyleActionSheet];

  UIAlertAction *logOutAction =
      [UIAlertAction actionWithTitle:@"退出登录"
                               style:UIAlertActionStyleDestructive
                             handler:^(UIAlertAction *_Nonnull action) {
                               [[IMSAccountService sharedService] logout];
                               [self updateLoginStatus];
                             }];

  UIAlertAction *cancelAction =
      [UIAlertAction actionWithTitle:@"取消"
                               style:UIAlertActionStyleCancel
                             handler:^(UIAlertAction *_Nonnull action) {
                               [self updateLoginStatus];
                             }];
  [actionSheetController addAction:logOutAction];
  [actionSheetController addAction:cancelAction];
  [self presentViewController:actionSheetController
                     animated:YES
                   completion:nil];
}

#pragma mark - UI creation

- (void)formatUI {
  [self.navigationController.navigationBar
      setBackgroundImage:[UIImage new]
           forBarMetrics:UIBarMetricsDefault];
  [self.navigationController.navigationBar setShadowImage:[UIImage new]];
  self.tableView.backgroundColor = [UIColor colorWithRed:239 / 255.0
                                                   green:238 / 255.0
                                                    blue:241 / 255.0
                                                   alpha:1.0];
  self.navigationController.navigationBar.titleTextAttributes = [NSDictionary
      dictionaryWithObjectsAndKeys:[UIFont fontWithName:@"PingFangSC-Regular"
                                                   size:18],
                                   NSFontAttributeName, nil];
  self.navigationController.navigationBar.translucent = NO;
  self.tableView.tableFooterView = [[UIView alloc] init];
  UIBarButtonItem *item =
      [[UIBarButtonItem alloc] initWithTitle:@""
                                       style:UIBarButtonItemStylePlain
                                      target:nil
                                      action:nil];
  self.navigationItem.backBarButtonItem = item;
  self.navigationController.navigationBar.tintColor = [UIColor blackColor];
}

- (void)createAPILevel {
  UIView *header = [[UIView alloc]
      initWithFrame:CGRectMake(0, 0, self.tableView.frame.size.width, 32)];
  self.tableView.tableHeaderView = header;
  header.backgroundColor = [UIColor colorWithRed:0 / 255.0
                                           green:0 / 255.0
                                            blue:0 / 255.0
                                           alpha:0.3 / 1.0];
  UILabel *label = [[UILabel alloc] init];
  label.frame = CGRectMake(28, 8, 194, 17);
  label.text = @"当前API Level : 1";
  label.font = [UIFont fontWithName:@"PingFangSC-Regular" size:12];
  label.textColor = [UIColor whiteColor];
  [header addSubview:label];
  UIImageView *hint =
      [[UIImageView alloc] initWithFrame:CGRectMake(9.5, 8, 16, 16)];
  [header addSubview:hint];
  [hint setImage:[UIImage imageNamed:@"hint"]];
}

- (void)createLogin {
  UIButton *image = [[UIButton alloc] initWithFrame:CGRectMake(10, 26, 28, 28)];
  [image setBackgroundImage:[UIImage imageNamed:@"quan"]
                   forState:UIControlStateNormal];
  [image addTarget:self
                action:@selector(loginEvent)
      forControlEvents:UIControlEventTouchUpInside];
  self.userImage = [[UIImageView alloc] initWithFrame:CGRectMake(6, 6, 16, 16)];
  [self.userImage setImage:[UIImage imageNamed:@"login"]];
  [image addSubview:self.userImage];

  if (@available(iOS 9, *)) {
    [image.widthAnchor constraintEqualToConstant:28].active = YES;
    [image.heightAnchor constraintEqualToConstant:28].active = YES;
  }

  UIBarButtonItem *myButton =
      [[UIBarButtonItem alloc] initWithCustomView:image];
  self.navigationItem.leftBarButtonItem = myButton;
}

- (void)updateLoginStatus {
  if ([[IMSAccountService sharedService] isLogin]) {
    self.userImage.image = [UIImage imageNamed:@"logined"];
  } else {
    self.userImage.image = [UIImage imageNamed:@"login"];
  }
}

- (void)createFloatPanel {
  UIButton *floatPanel = [[UIButton alloc]
      initWithFrame:CGRectMake(self.view.frame.size.width - 18 - 49,
                               [UIScreen mainScreen].bounds.size.height - 27 -
                                   137,
                               49, 137)];
  [floatPanel.layer setMasksToBounds:YES];
  [floatPanel.layer setCornerRadius:6.0];

  floatPanel.backgroundColor = [UIColor colorWithRed:35 / 255.0
                                               green:35 / 255.0
                                                blue:35 / 255.0
                                               alpha:0.6 / 1.0];
  [self.navigationController.view addSubview:floatPanel];
  UIButton *scanClick =
      [[UIButton alloc] initWithFrame:CGRectMake(12, 12, 24, 24)];
  [scanClick addTarget:self
                action:@selector(goScan)
      forControlEvents:UIControlEventTouchUpInside];

  UIButton *logClick =
      [[UIButton alloc] initWithFrame:CGRectMake(12, 57, 24, 24)];
  [logClick addTarget:self
                action:@selector(goLog)
      forControlEvents:UIControlEventTouchUpInside];

  UIButton *aboutClick =
      [[UIButton alloc] initWithFrame:CGRectMake(12, 57 + 21 + 24, 24, 24)];
  [aboutClick addTarget:self
                 action:@selector(goAbout)
       forControlEvents:UIControlEventTouchUpInside];

  [floatPanel addSubview:scanClick];
  [floatPanel addSubview:logClick];
  [floatPanel addSubview:aboutClick];
  [aboutClick setBackgroundImage:[UIImage imageNamed:@"about"]
                        forState:UIControlStateNormal];
  [logClick setBackgroundImage:[UIImage imageNamed:@"log"]
                      forState:UIControlStateNormal];
  [scanClick setBackgroundImage:[UIImage imageNamed:@"scan"]
                       forState:UIControlStateNormal];

  [floatPanel addTarget:self
                 action:@selector(dragMoving:withEvent:)
       forControlEvents:UIControlEventTouchDragInside];
}




@end

