//
//  BoneDemoProxy.m
//  BoneKitDemo
//
//  Created by dujin on 2017/11/19.
//  Copyright © 2017年 Alibaba. All rights reserved.
//

#import "BoneDemoProxy.h"

#import <IMSBoneKit/BoneRCTViewController.h>
#import <IMSRouter/IMSRouter.h>
#import <IMSHUD/UIViewController+HUD.h>
#import <IMSPackage/IMSPackageManager.h>

static NSString * const KLastBoneDebugIPKey = @"LastBoneDebugIPKey";
static NSString * const KBonePluginEnvironmentKey = @"KBonePluginEnvironmentKey";

@interface BoneDemoProxy ()

@property (nonatomic, weak) UIViewController *controller;
@property (nonatomic, strong) NSString *errorMsg;

@end

@implementation BoneDemoProxy

+ (id)proxyWithController:(UIViewController *)controller {
    static BoneDemoProxy *instance;
    static dispatch_once_t oncePredicate;
    
    dispatch_once(&oncePredicate, ^{
        instance = [[BoneDemoProxy alloc] init];
        instance.controller = controller;
    });
    
    return instance;
}

#pragma mark - Proxy

- (void)showBoneApiDemo {
    [[IMSRouterService sharedService] openURL:[NSURL URLWithString:@"link://a123Exz5WyvOBQrX"] options:nil];
}


- (void)showBoneUIDemo {
    [[IMSRouterService sharedService] openURL:[NSURL URLWithString:@"link://a123XrP57FLvs4u2"] options:nil];
}

- (void)showBoneEnvDemo {
    __weak typeof(self) weakSelf = self;
    UIAlertAction *testAction = [UIAlertAction actionWithTitle:@"开发环境" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [weakSelf setBoneRunEnvironment:@"test"];
    }];
    UIAlertAction *releaseAction = [UIAlertAction actionWithTitle:@"生产环境" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [weakSelf setBoneRunEnvironment:@"release"];
    }];
    
    UIAlertController *controller = [UIAlertController alertControllerWithTitle:@"环境切换" message:nil preferredStyle:UIAlertControllerStyleActionSheet];
    [controller addAction:testAction];
    [controller addAction:releaseAction];
    [controller addAction:[UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:nil]];
    
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wundeclared-selector"
    if ([releaseAction respondsToSelector:@selector(_setTitleTextColor:)]) {
        [releaseAction performSelector:@selector(_setTitleTextColor:) withObject:[UIColor colorWithRed:51.0/255.0 green:51.0/255.0 blue:51.0/255.0 alpha:1.0]];
    }
    if ([testAction respondsToSelector:@selector(_setTitleTextColor:)]) {
        [testAction performSelector:@selector(_setTitleTextColor:) withObject:[UIColor colorWithRed:51.0/255.0 green:51.0/255.0 blue:51.0/255.0 alpha:1.0]];
    }
#pragma clang diagnostic pop
    
    [self.controller presentViewController:controller animated:YES completion:nil];
}

- (void)showBoneDebug {
    NSString *lastDebugIP = [[NSUserDefaults standardUserDefaults] objectForKey:KLastBoneDebugIPKey];
    
    __weak typeof(self) wSelf = self;
    UIAlertController *controller = [UIAlertController alertControllerWithTitle:nil
                                                                        message:@"请输入ip地址"
                                                                 preferredStyle:UIAlertControllerStyleAlert];
    [controller addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = @"ip address";
        textField.text = lastDebugIP ? : @"";
    }];
    [controller addAction:[UIAlertAction actionWithTitle:@"确认" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        NSString *host = [controller.textFields.firstObject.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
        if (host.length > 0) {
            self.errorMsg = @"无效的ip地址";
            [wSelf executeDebugWithHost:host];
        }
    }]];
    [controller addAction:[UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) { }]];
    [self.controller presentViewController:controller animated:true completion:nil];
}

- (void)showBoneRouter {
    [[IMSRouterService sharedService] openURL:[NSURL URLWithString:@"plugin://F3C2C66C449C68BE3BB3A9FD023F59FC#/"] options:@{@"key0": @"value0"}];
}

- (void)testOfflinePackage {
    [[IMSPackageManager defaultManager] deleteOfflinePackage];
    [[IMSPackageManager defaultManager] fetchOfflinePackageUpdate:nil];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        NSURL *URL = [NSURL URLWithString:@"http://g.alicdn.com/aic/mobile-sdk-demo/1.0.5/app.js"];
        NSURLRequest *request = [NSURLRequest requestWithURL:URL];
        NSURLSession *session = [NSURLSession sharedSession];
        NSURLSessionDownloadTask *downloadTask = [session downloadTaskWithRequest:request
                                                                completionHandler:^(NSURL *location, NSURLResponse *response, NSError *error)
                                                  {
                                                      if (response && !error) {
                                                          NSLog(@"hit: %@", location);
                                                      }
                                                  }];
        
        [downloadTask resume];
    });
}

- (void)handleBoneQRCode:(NSString *)urlString {
    NSURL *url = [NSURL URLWithString:urlString];
    NSDictionary *para= [BoneDemoProxy bone_queryStringToDictionary:url.query];
    NSString *encodeIP = [para valueForKey:@"boneDebugHost"];
    NSString *decodeIP = [encodeIP stringByRemovingPercentEncoding];
    if (decodeIP.length > 0) {
        self.errorMsg = @"扫码失败！请检查二维码是否正确!";
        [self executeDebugWithHost:decodeIP];
    }
}

#pragma mark - Private

- (void)executeDebugWithHost:(NSString *)host {
    if (host.length == 0) {
        return;
    }
    
    __weak typeof(self) wSelf = self;
    [[self controller] ims_showHUDWithIndicator];
    
    NSURLSessionConfiguration *configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
    configuration.timeoutIntervalForRequest = 15;
    NSURLSession *session = [NSURLSession sessionWithConfiguration:configuration];
    
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://%@:8081/boneDebugUrl?platform=ios&ip=%@", host, host]]];
    NSURLSessionDataTask *task = [session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [[wSelf controller] ims_hideHUD];
            [[NSUserDefaults standardUserDefaults] setObject:host forKey:KLastBoneDebugIPKey];
            
            if (!error) {
                NSError *jsonError = nil;
                NSDictionary *responseJSON = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:&jsonError];
                if (jsonError) {
                    self.errorMsg = @"返回数据格式错误";
                    [wSelf showErrorAlert:jsonError];
                    return;
                }
                
                int responseCode = [[responseJSON objectForKey:@"code"] intValue];
                if ( responseCode == 200) {
                    NSDictionary *responseData= [responseJSON objectForKey:@"data"];
                    NSString *urlString = responseData[@"url"];
                    BoneRCTViewController *rnController = [BoneRCTViewController new];
                    [rnController openUrl:[NSURL URLWithString:urlString] props:[responseData mutableCopy]];
                    rnController.hidesBottomBarWhenPushed = YES;
                    [[self controller].navigationController pushViewController:rnController animated:YES];
                    return;
                }
            }
            [wSelf showErrorAlert:error];
        });
    }];
    [task resume];
}

- (void)showErrorAlert:(NSError *)error {
    UIAlertController *controller = [UIAlertController alertControllerWithTitle:nil message:self.errorMsg preferredStyle:UIAlertControllerStyleAlert];
    [controller addAction:[UIAlertAction actionWithTitle:@"确认" style:UIAlertActionStyleDefault handler:nil]];
    [[self controller] presentViewController:controller animated:true completion:nil];
}

- (void)setBoneRunEnvironment:(NSString *)env {
    NSString *curEnv = [BoneDemoProxy getPluginEnvironment];
    if ([curEnv isEqualToString:env]) {
        NSString *msg = [env isEqualToString:@"test"] ? @"开发" : @"生产";
        [[self controller] ims_showHUDWithMessage:[NSString stringWithFormat:@"当前已经是%@环境", msg]];
    } else {
        [BoneDemoProxy setPluginEnvironment:env];
        
        [[self controller] ims_showHUDWithMessage:@"即将重启程序\n请稍后重新登录"];
        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^(void){
            exit(0);
        });
    }
}

#pragma mark - Environment

+ (NSString *)getPluginEnvironment {
    return [[NSUserDefaults standardUserDefaults] objectForKey:KBonePluginEnvironmentKey];
}

+ (void)setPluginEnvironment:(NSString *)environment {
    if ([@[@"test", @"release"] containsObject:environment]) {
        [[NSUserDefaults standardUserDefaults] setObject:environment forKey:KBonePluginEnvironmentKey];
        [[NSUserDefaults standardUserDefaults] synchronize];
    }
}

#pragma mark - URL

+ (NSMutableDictionary *)bone_queryStringToDictionary:(NSString *)string {
    NSArray *queryItemStrings = [string componentsSeparatedByString:@"&"];
    
    NSMutableDictionary *queryDictionary = [NSMutableDictionary dictionaryWithCapacity:[queryItemStrings count]];
    for(NSString *queryItemString in queryItemStrings) {
        NSRange range = [queryItemString rangeOfString:@"="];
        if (range.location != NSNotFound) {
            NSString *key = [BoneDemoProxy bone_decode:[queryItemString substringToIndex:range.location]];
            NSString *value = [BoneDemoProxy bone_decode:[queryItemString substringFromIndex:range.location + 1]];
            [queryDictionary setObject:value forKey:key];
        }
    }
    return queryDictionary;
}

+ (NSString *)bone_decode:(NSString *)s {
    if (!s) return nil;
    return CFBridgingRelease(CFURLCreateStringByReplacingPercentEscapes(NULL, (CFStringRef)s, CFSTR("")));
}

@end
