//
//  AccountViewController.m
//  IMSAccountDemo
//
//  Created by Hager Hu on 02/11/2017.
//  Copyright © 2017 Aliyun.com. All rights reserved.
//

#import "AccountViewController.h"

#import <IMSHUD/UIViewController+HUD.h>
#import <IMSAccount/IMSAccountService.h>

@interface AccountViewController ()

@property (nonatomic, weak) IBOutlet UIButton *loginButton;
@property (nonatomic, weak) IBOutlet UIButton *logoutButton;

@end

@implementation AccountViewController

+ (UIViewController *)storyboardController {
    NSBundle *bundle = [NSBundle bundleForClass:[self class]];
    NSURL *url = [bundle URLForResource:@"IMSAccountDemo" withExtension:@".bundle"];
    NSBundle *demoBundle = [NSBundle bundleWithURL:url];
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"Main" bundle:demoBundle];
    NSString *name = NSStringFromClass([self class]);
    
    return [storyboard instantiateViewControllerWithIdentifier:name];
}


#pragma mark -

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    
    UIColor *textColor = [self.loginButton titleColorForState:UIControlStateNormal];
    self.loginButton.layer.borderColor = [textColor CGColor];
    self.loginButton.layer.borderWidth = 1.0f;
    self.loginButton.layer.cornerRadius = 5.0f;
    
    textColor = [self.logoutButton titleColorForState:UIControlStateNormal];
    self.logoutButton.layer.borderColor = [textColor CGColor];
    self.logoutButton.layer.borderWidth = 1.0f;
    self.logoutButton.layer.cornerRadius = 5.0f;
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - Handle Button Events

- (IBAction)loginButtonDidClicked:(id)sender {
    [[IMSAccountService sharedService] showLoginWithController:self success:^(NSDictionary *userInfo) {
        [self ims_showHUDWithMessage:@"登录成功"];
    } failure:^(NSError *error) {
        NSString *messsage = [NSString stringWithFormat:@"%ld\n%@", error.code, error.localizedDescription];
        [self ims_showHUDWithMessage:messsage];
    }];
}


- (IBAction)logoutButtonDidClicked:(id)sender {
    if ([[IMSAccountService sharedService] isLogin]) {
        [[IMSAccountService sharedService] logout];
        
        [self ims_showHUDWithMessage:@"退出成功"];
    } else {
        [self ims_showHUDWithMessage:@"当前没有登录"];
    }
}

@end
