//
//  IMSApiClientDemoViewController.m
//  Pods-IMSAppDemo
//
//  Created by 文季 on 2017/11/8.
//

#import "IMSApiClientDemoViewController.h"
#import <IMSApiClient/IMSApiClient.h>
#import <IMSLog/IMSLog.h>
#import <IMSHUD/UIViewController+HUD.h>
#import <UITextView+Placeholder/UITextView+Placeholder.h>
#import "IMSApiClientResultViewController.h"

@interface IMSApiClientDemoViewController () <UITextFieldDelegate, UIGestureRecognizerDelegate>

@property (weak, nonatomic) IBOutlet UITextField *hostTextField;
@property (weak, nonatomic) IBOutlet UITextField *pathTextField;
@property (weak, nonatomic) IBOutlet UITextField *apiVersionTextField;
@property (weak, nonatomic) IBOutlet UITextView *paramsTextView;

@property (weak, nonatomic) IBOutlet UIButton *httpButton;
@property (weak, nonatomic) IBOutlet UIButton *httpsButton;

@property (weak, nonatomic) IBOutlet UIButton *startTestButton;

@property (nonatomic, strong) NSArray *apiList;

@property (nonatomic, strong) UIGestureRecognizer *tapGesture;

@end

@implementation IMSApiClientDemoViewController

+ (UIViewController *)storyboardController {
    NSBundle *bundle = [NSBundle bundleForClass:[self class]];
    NSURL *url = [bundle URLForResource:@"IMSApiClientDemo" withExtension:@".bundle"];
    if (!url) {
        url = [bundle bundleURL];
    }
    NSBundle *demoBundle = [NSBundle bundleWithURL:url];
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"IMSApiClientDemo" bundle:demoBundle];
    NSString *name = NSStringFromClass([self class]);
    
    return [storyboard instantiateViewControllerWithIdentifier:name];
}

- (void)loadWithJson:(NSDictionary *)json {
    self.hostTextField.text = json[@"host"] ?: [IMSConfiguration sharedInstance].host;
    self.pathTextField.text = json[@"path"] ?: @"";
    self.apiVersionTextField.text = json[@"apiVersion"] ?: @"";
    NSDictionary *params = json[@"params"] ?: @{};
    if (![params isKindOfClass:[NSDictionary class]]) {
        params = @{};
    }
    NSData *data = [NSJSONSerialization dataWithJSONObject:params options:NSJSONWritingPrettyPrinted error:nil];
    NSString *str = @"{}";
    if (data) {
        str = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    }
    self.paramsTextView.text = str ?: @"{}";
    NSString *scheme = json[@"scheme"] ?: @"";
    self.httpButton.selected = [scheme isEqualToString:@"http"];
    self.httpsButton.selected = [scheme isEqualToString:@"https"];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    NSBundle *demoBundle = [self demoBundle];
    NSString *path = [NSString pathWithComponents:@[[demoBundle bundlePath] ?: @"", @"api_list.json"]];
    
    NSData *data = [NSData dataWithContentsOfFile:path];
    if (data) {
        NSDictionary *json = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
        if (json && [json isKindOfClass:[NSDictionary class]]) {
            self.apiList = [json objectForKey:@"list"];
        }
    }
    
    self.httpsButton.selected = YES;
    self.httpButton.selected = NO;
    
    self.paramsTextView.placeholder = @"请输入JSON格式参数";
    self.hostTextField.delegate = self;
    self.pathTextField.delegate = self;
    self.apiVersionTextField.delegate = self;

    [self.hostTextField setValue:[UIColor colorWithRed:153 / 255.0
                                                 green:153 / 255.0
                                                  blue:153 / 255.0
                                                 alpha:1 / 1.0]
                      forKeyPath:@"_placeholderLabel.textColor"];
    [self.pathTextField setValue:[UIColor colorWithRed:153/255.0 green:153/255.0 blue:153/255.0 alpha:1/1.0] forKeyPath:@"_placeholderLabel.textColor"];
    [self.apiVersionTextField setValue:[UIColor colorWithRed:153/255.0 green:153/255.0 blue:153/255.0 alpha:1/1.0] forKeyPath:@"_placeholderLabel.textColor"];
    [self.paramsTextView setPlaceholderColor:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:1/1.0]];
    
    
    
    _tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(gestureRecognized)];
    [self.tableView addGestureRecognizer:self.tapGesture];
    self.tableView.backgroundColor=[UIColor colorWithRed:239.0 / 255
                                                   green:238.0 / 255
                                                    blue:241.0 / 255
                                                   alpha:1];
    self.tableView.separatorColor=[UIColor colorWithRed:234 / 255.0
                                                  green:234 / 255.0
                                                   blue:234 / 255.0
                                                  alpha:1 / 1.0];
    

}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)alertWithMsg:(NSString *)message {
    UIAlertController *controller = [UIAlertController alertControllerWithTitle:nil message:message preferredStyle:UIAlertControllerStyleAlert];
    [controller addAction:[UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleCancel handler:nil]];
    [self.navigationController presentViewController:controller animated:YES completion:nil];
}


#pragma mark - UITableViewDelegate

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    if (section == 0) {
        return 20.0f;
    }
    
    if (section == 3 || section == 4) {
        return 30.0f;
    }
    
    return CGFLOAT_MIN;
}


- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section {
    if (section == 2 || section == 3) {
        return CGFLOAT_MIN;
    }
    
    if (section == 4) {
        return 20.0f;
    }
    
    return 10.0f;
}

- (void)tableView:(UITableView *)tableView
    willDisplayHeaderView:(UIView *)view
               forSection:(NSInteger)section {
  // Text Color
  UITableViewHeaderFooterView *header = (UITableViewHeaderFooterView *)view;
  [header.textLabel setTextColor:[UIColor colorWithRed:155 / 255.0
                                                 green:155 / 255.0
                                                  blue:155 / 255.0
                                                 alpha:1 / 1.0]];
  header.textLabel.font = [UIFont fontWithName:@"PingFangSC-Regular" size:12];
}


#pragma mark - UIGestureRecognizerDelegate

- (void)gestureRecognized {
    [self.view endEditing:YES];
}


#pragma mark - Handle Event

- (IBAction)onStartFillData:(UIBarButtonItem *)sender {
    
    if ([self.apiList count] >= 1) {
        self.json = self.apiList[0];
    }
    
    [self loadWithJson:self.json];
}


- (IBAction)onStartTest:(UIButton *)sender {
    NSString *path = self.pathTextField.text;
    if ([path length] == 0) {
        [self alertWithMsg:@"path不能为空"];
        return;
    }
    NSString *apiVersion = self.apiVersionTextField.text;
    if ([apiVersion length] == 0) {
        [self alertWithMsg:@"apiVersion不能为空"];
        return;
    }
    
    if ([self.paramsTextView.text length] == 0) {
        self.paramsTextView.text = @"{}";
    }
    
    NSData *data = [self.paramsTextView.text dataUsingEncoding:NSUTF8StringEncoding];
    if (data == nil) {
        [self alertWithMsg:@"JSON参数不合法"];
        return;
    }
    NSError *error;
    NSDictionary *params = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
    if (error) {
        NSString *msg = [NSString stringWithFormat:@"JSON参数不合法：%@", error];
        [self alertWithMsg:msg];
        return;
    }
    
    [self ims_showHUDWithMessage:@"正在发送请求..."];
    IMSIoTRequestBuilder *builder = [[IMSIoTRequestBuilder alloc] initWithPath:path
                                                                    apiVersion:apiVersion
                                                                        params:params];
    NSString *host = self.hostTextField.text;
    if (host && [host length] > 0) {
        [builder setHost:host];
    }
    [builder setScheme:(self.httpButton.selected) ? @"http" : @"https"];
    self.startTestButton.enabled = NO;
    [IMSRequestClient asyncSendRequest:builder.build responseHandler:^(NSError * _Nullable error, IMSResponse * _Nullable response) {
        [self ims_hideHUD];
        
        self.startTestButton.enabled = YES;
        
        NSString *result = @"";
        if (error) {
            result = [NSString stringWithFormat:@"%@", error];
        } else {
            id rawData = response.rawData;
            if ([response.rawData isKindOfClass:[NSData class]]) {
                //转换服务端应答原始数据为字符串，可用于排查问题
                NSString *str = [[NSString alloc] initWithData:rawData encoding:NSUTF8StringEncoding];
                if (str) {
                    rawData = str;
                }
            }
            NSString *responeStr = [NSString stringWithFormat:@"code:\n%@\n\ndata:\n%@\n\nmessage:\n %@\n\nlocalizedMsg:\n%@\n\nrawData:\n%@",
                                    @(response.code),
                                    response.data,
                                    response.message,
                                    response.localizedMsg,
                                    rawData ?: @""];
            result = responeStr;
        }
        
        IMSApiClientResultViewController *controller = [IMSApiClientResultViewController storyboardController];
        controller.result = result;
        
        [self.navigationController pushViewController:controller animated:YES];
    }];
}

#pragma mark - delegate

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldEndEditing:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldClear:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

#pragma mark - Handling UIButton

- (IBAction)onClickHttpCell:(id)sender {
    self.httpButton.selected = YES;
    self.httpsButton.selected = NO;
    [self.tableView reloadData];
}

- (IBAction)onClickHttpsCell:(id)sender {
    self.httpButton.selected = NO;
    self.httpsButton.selected = YES;
    [self.tableView reloadData];
}

#pragma mark -

- (NSBundle *)demoBundle {
    NSBundle *bundle = [NSBundle bundleForClass:[self class]];
    NSURL *url = [bundle URLForResource:@"IMSApiClientDemo" withExtension:@".bundle"];
    if (!url) {
        url = [bundle bundleURL];
    }
    
    return [NSBundle bundleWithURL:url];
}

@end
