//
//  AKDebugDashboard.m
//  AlinkApp
//
//  Created by Wenji.lsl on 15/11/25.
//  Copyright © 2015年 Alink. All rights reserved.
//

#import "AKDebugDashboard.h"
#import "AKLogView.h"
#import "PTEConsoleTableView.h"
#import "PTEConsoleLogger.h"
#import "IMSLogSettingView.h"

#undef NSLog
#import <UIKit/UIKit.h>

#define SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(v)  ([[[UIDevice currentDevice] systemVersion] compare:v options:NSNumericSearch] != NSOrderedAscending)

#define LOGO_WINDOW_HEIGHT 20
#define STATUSBAR_HEIGHT ([[UIApplication sharedApplication] statusBarFrame].size.height + 44)
#define SCOPE_BAR_HEIGHT 28

#pragma mark - logo window

@interface AKDebugLogoWindowRootViewController : UIViewController

@end

@implementation AKDebugLogoWindowRootViewController

- (BOOL)prefersStatusBarHidden{
	return NO;
}

@end

@interface AKDebugLogoWindow : UIWindow

- (instancetype)initWithFrame:(CGRect)frame;
- (void)becomeKeyWindow;

@end

@implementation AKDebugLogoWindow

- (instancetype)initWithFrame:(CGRect)frame
{
	self = [super initWithFrame:frame];
	
	if (self) {
		//每个UIWindow都需要一个rootViewController
		//添加空的rootViewController
		//修正*** Assertion failure in -[UIApplication _runWithMainScene:transitionContext:completion:]错误
		self.rootViewController = [[AKDebugLogoWindowRootViewController alloc] init];
		self.rootViewController.view.hidden = YES;
	}
	
	return self;
}

- (void)becomeKeyWindow
{
	[[UIApplication sharedApplication].delegate.window makeKeyWindow];
}

@end

#pragma mark - debug dashboard

@interface AKDebugDashboard () <AKLogViewDelegate>

//logo
@property (nonatomic, strong) UIWindow *logoWindow;
@property (nonatomic, strong) UIButton *logoButton;

//ScopeBar
@property (nonatomic, weak) IBOutlet UIView *contentView;
@property (nonatomic, strong) IBOutlet UIScrollView *scrollView;
@property (nonatomic, strong) IBOutlet UISegmentedControl *segmentedControl;
@property (nonatomic, strong) NSMutableArray *controlViews;
@property(nonatomic, strong) UIButton * closeButton;
@property(nonatomic, strong) AKLogView *logView;
- (IBAction)segmentIndexChanged:(UISegmentedControl *)sender;

@end

@implementation AKDebugDashboard

#pragma mark - 生命周期

+ (instancetype)sharedDashboard
{
	static AKDebugDashboard *_sharedDashboard;
	static dispatch_once_t onceToken;
	
	dispatch_once(&onceToken, ^{
		_sharedDashboard = [[[NSBundle mainBundle] loadNibNamed:@"AKDebugDashboard" owner:nil options:nil] firstObject];
	});
	
	return _sharedDashboard;
}

- (void)awakeFromNib
{
	[super awakeFromNib];
	// 日志面板状态栏开关
//    [self loadLogo];
	[self loadUI];
	[self loadDefaultLogView];
}

- (void)loadUI
{

    
	//设置控制面板
	CGRect screenFrame = [UIScreen mainScreen].bounds;
	self.frame = CGRectMake(0, 0,
							screenFrame.size.width,
							screenFrame.size.height);
	
	[self.segmentedControl removeAllSegments];
    
    [self customnavigationView];
    
//    UINavigationBar * navBar = [[UINavigationBar alloc]initWithFrame:CGRectMake(0, [[UIApplication sharedApplication] statusBarFrame].size.height, screenFrame.size.width, 44)];
//    [self addSubview:navBar];
//
//    UIButton * closeButton= [UIButton buttonWithType:UIButtonTypeSystem];
//    closeButton.frame =  CGRectMake(15, STATUSBAR_HEIGHT- 36, 30, 30);;
//    [closeButton setTitle:@"✕" forState:UIControlStateNormal];
//    closeButton.titleLabel.font = [UIFont systemFontOfSize:25];
//    [closeButton addTarget:self action:@selector(closeButtonTouched:) forControlEvents:UIControlEventTouchUpInside];
//    UIBarButtonItem * leftBarButtonItem = [[UIBarButtonItem alloc]initWithCustomView:closeButton];
//
//    UIBarButtonItem * rightItem = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemAction target:self action:@selector(shareButtonTouched:)];
    
   
    
}
-(void)customnavigationView{
    CGRect screenFrame = [UIScreen mainScreen].bounds;
    
    UIView * navView =[[UIView alloc]init];
    navView.backgroundColor = [UIColor groupTableViewBackgroundColor];
    navView.frame = CGRectMake(0, 0, screenFrame.size.width, STATUSBAR_HEIGHT);
    
    
    UILabel *label = [[UILabel alloc] init];
    label.frame = CGRectMake(screenFrame.size.width/2 - 20, STATUSBAR_HEIGHT- 35,
                             40,
                             25);
    label.text = @"日志";
    label.textColor = [UIColor blackColor];
    [navView addSubview:label];
    
    
    UIButton * closeButton= [UIButton buttonWithType:UIButtonTypeSystem];
    closeButton.frame = CGRectMake(screenFrame.size.width-45, STATUSBAR_HEIGHT- 36, 30, 30);
    [closeButton setTitle:@"✕" forState:UIControlStateNormal];
    closeButton.titleLabel.font = [UIFont systemFontOfSize:25];
    [closeButton addTarget:self action:@selector(closeButtonTouched:) forControlEvents:UIControlEventTouchUpInside];
    [navView addSubview:closeButton];
    
    
    UIButton * shareButton= [UIButton buttonWithType:UIButtonTypeSystem];
    shareButton.frame = CGRectMake(15, STATUSBAR_HEIGHT- 36, 30, 30);
    [shareButton setTitle:@"⤴︎" forState:UIControlStateNormal];
    shareButton.titleLabel.font = [UIFont systemFontOfSize:25];
    [shareButton addTarget:self action:@selector(shareButtonTouched:) forControlEvents:UIControlEventTouchUpInside];
    [navView addSubview:shareButton];
    
    [self addSubview:navView];
}
-(void)shareButtonTouched:(UIButton *)button{

    NSArray * logMessages = self.logView.logger.logMessages;
    NSString * fullLogStr =@"";
    for (DDLogMessage * logMessage in logMessages) {
        NSString *timestampStr = [[self.logView.logger.class dateFormatter] stringFromDate:logMessage.timestamp];
        NSString * messageString = [NSString stringWithFormat:@"%@ | %@ - %@:%lu | %@",timestampStr,logMessage.tag,logMessage.fileName,(unsigned long)logMessage.line,logMessage.message];
        
        fullLogStr = [fullLogStr stringByAppendingString:messageString];
        fullLogStr = [fullLogStr stringByAppendingString:@"\n"];
    }
    
    NSString *path = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES).firstObject;
    NSString *fileName = [path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.txt",[self getTodayDateStr]]];
    NSURL * fileUrl = [NSURL fileURLWithPath:fileName];
    
    BOOL  isWrite = [fullLogStr writeToURL:fileUrl atomically:YES encoding:NSUTF8StringEncoding error:nil];
    if (!isWrite) {
        return;
    }

    UIActivityViewController *activityViewController = [[UIActivityViewController alloc] initWithActivityItems:@[@"日志", [NSURL fileURLWithPath:fileName]] applicationActivities:nil];
    
    [[UIApplication sharedApplication].keyWindow.rootViewController  presentViewController:activityViewController animated:YES completion:nil];
    [self setFullScreen:NO];
}

-(NSString*)getTodayDateStr//获得当前时间（日期）
{
    NSDateFormatter  *dateformatter=[[NSDateFormatter alloc] init];
    
    [dateformatter setDateFormat:@"YYYY-MM-dd-HH-mm"];
    
    return [dateformatter stringFromDate:[NSDate date]];
}

-(void)closeButtonTouched:(UIButton *)button{
    [self setFullScreen:NO];
}
- (void)loadLogo
{
	//添加logoWindow
    CGFloat offset = 100;
    if ([UIApplication sharedApplication].statusBarFrame.size.height > 20) {
        offset = 15.0f;
    }
	_logoWindow = [[AKDebugLogoWindow alloc] initWithFrame:CGRectMake(offset, 0,
																68, LOGO_WINDOW_HEIGHT)];
	_logoWindow.windowLevel = UIWindowLevelStatusBar + 1;
	_logoWindow.backgroundColor = [UIColor blackColor];
	
	//添加logoButton
	_logoButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
	_logoButton.frame = _logoWindow.bounds;
	[_logoButton setTitle:@"日志" forState:UIControlStateNormal];
	[_logoButton addTarget:self action:@selector(logoButtonPress:) forControlEvents:UIControlEventTouchUpInside];
	
	[_logoWindow addSubview:_logoButton];
	_logoWindow.hidden = NO;
}

- (void)loadDefaultLogView
{
	//添加日志面板
	self.logView = [AKLogView viewFromStoryboard];
	self.logView.delegate = self;
	[self insertSegmentWithTitle:@"全部日志" view:self.logView atIndex:0];
	[self selectSegmentAtIndex:0];

}

#pragma mark - layout

- (void)layoutSubviews
{
	[super layoutSubviews];
	
	//如果面板个数<=1个的时候隐藏scrollView和segmentedControl
	//segment的长度超过屏幕宽度需要加入滚动条
	
	CGFloat margin = 8;
	CGFloat contentViewTop = margin;
	NSUInteger numberOfSegments = self.segmentedControl.numberOfSegments;
	
	//scrollView
	if (numberOfSegments < 2) {
		[self.scrollView removeFromSuperview];
	}
	else {
		BOOL contain = NO;
		for (UIView *view in self.subviews) {
			if (view == self.scrollView) {
				contain = YES;
				break;
			}
		}
		
		if (!contain) {
			[self addSubview:self.scrollView];
		}
		
		self.scrollView.frame = CGRectMake(margin,
										   margin + STATUSBAR_HEIGHT,
										   self.bounds.size.width - margin * 2,
										   SCOPE_BAR_HEIGHT);
		contentViewTop = self.scrollView.bounds.size.height + margin * 2;
	}


	//segmentedControl and contentSize
	[self.segmentedControl sizeToFit];
	self.segmentedControl.frame = CGRectMake(0, 0,
											 MAX(self.segmentedControl.bounds.size.width, self.bounds.size.width - margin * 2),
											 self.scrollView.bounds.size.height);
	self.scrollView.contentSize = CGSizeMake(self.segmentedControl.bounds.size.width,
											 self.segmentedControl.bounds.size.height);
	
	//contentView
	self.contentView.frame = CGRectMake(margin,
										contentViewTop + STATUSBAR_HEIGHT,
										self.bounds.size.width - margin * 2,
										self.bounds.size.height - contentViewTop - STATUSBAR_HEIGHT);
	
	if (self.segmentedControl.selectedSegmentIndex < self.controlViews.count &&
		self.segmentedControl.selectedSegmentIndex >= 0) {
		UIView *selectedView = self.controlViews[self.segmentedControl.selectedSegmentIndex];
		selectedView.frame = self.contentView.bounds;
	}
}
 
#pragma mark - utils

- (void)setFullScreen:(BOOL)fullScreen
{
	if (fullScreen == _fullScreen) {
		return;
	}
	float  changeHeight = [UIScreen mainScreen].bounds.size.height;
	if (fullScreen) {

        self.center = CGPointMake(self.center.x, self.center.y + changeHeight);
        [[UIApplication sharedApplication].delegate.window addSubview:self];
        [UIView animateWithDuration:0.25 delay:0 options:UIViewAnimationOptionCurveEaseInOut animations:^{
              self.center = CGPointMake(self.center.x, self.center.y - changeHeight);
        } completion:nil];
        
	}
	else {
        [UIView animateWithDuration:0.25 delay:0 options:UIViewAnimationOptionCurveEaseInOut animations:^{
            self.center = CGPointMake(self.center.x, self.center.y + changeHeight);
        } completion:^(BOOL finished) {
             [self removeFromSuperview];
             self.center = CGPointMake(self.center.x, self.center.y - changeHeight);
        }];
       
	}
	
	_fullScreen = fullScreen;
}

- (void)logoButtonPress:(id)sender
{
	[self setFullScreen:!_fullScreen];
}

#pragma mark - 组件面板

- (NSMutableArray *)controlViews
{
	if (_controlViews) {
		return _controlViews;
	}
	
	return (_controlViews = [[NSMutableArray alloc] initWithCapacity:5]);
}

- (NSUInteger)numberOfSegments
{
	return self.segmentedControl.numberOfSegments;
}

- (UIView *)selectSegmentAtIndex:(NSUInteger)segment
{
	if (segment >= [self.controlViews count]) {
		
		return nil;
	}
	
	self.segmentedControl.selectedSegmentIndex = segment;
	UIView *selectedView = self.controlViews[segment];
	if (self.contentView.subviews &&
		self.contentView.subviews.firstObject == selectedView) {
			return selectedView;
	}
	
	for (UIView *view in self.contentView.subviews) {
		[view removeFromSuperview];
	}
	
	selectedView.frame = self.contentView.bounds;
	[self.contentView addSubview:selectedView];
	return selectedView;
}

- (void)insertSegmentWithTitle:(NSString *)title view:(UIView *)view atIndex:(NSUInteger)segment
{
	if (segment > self.controlViews.count) {
		return;
	}
	
	[self.segmentedControl insertSegmentWithTitle:title atIndex:segment animated:NO];
	[self.controlViews insertObject:view atIndex:segment];
	
	[self setNeedsLayout];
}

- (void)removeSegmentAtIndex:(NSUInteger)segment
{
	if (segment >= self.controlViews.count) {
		return;
	}
	
	NSUInteger oldSelectSegment = self.segmentedControl.selectedSegmentIndex;
	[self.segmentedControl removeSegmentAtIndex:segment animated:NO];
	[self.controlViews removeObjectAtIndex:segment];
	
	//如果移除的是当前选择的segment，则向前显示一个
	//如果全部移除了，则清空contentView
	if (oldSelectSegment == segment) {
		NSUInteger selectSegment = oldSelectSegment > 0 ? oldSelectSegment - 1 : 0;
		if (self.segmentedControl.numberOfSegments > 0) {
			[self selectSegmentAtIndex:selectSegment];
		}
		else {
			for (UIView *view in self.contentView.subviews) {
				[view removeFromSuperview];
			}
		}
	}
	
	[self setNeedsLayout];
}

- (IBAction)segmentIndexChanged:(UISegmentedControl *)sender
{
    if ([self.delegate respondsToSelector:@selector(dashboard:selectSegmentIndex:)]) {
        [self.delegate dashboard:self selectSegmentIndex:sender.selectedSegmentIndex];
    }
	[self selectSegmentAtIndex:sender.selectedSegmentIndex];
}

#pragma mark - delegate

- (void)logView:(AKLogView *)logView onUploadLogClicked:(NSArray *)messageArray
{
	if ([self.delegate respondsToSelector:@selector(dashboard:onUploadLogClicked:)]) {
		[self.delegate dashboard:self onUploadLogClicked:messageArray];
	}
}

@end
