//
//  IMSMobileChannelTableViewController.m
//  IMSMobileChannelDemo
//
//  Created by 文季 on 2017/11/16.
//  Copyright © 2017年 Alibaba-Inc. All rights reserved.
//

#import "IMSMobileChannelTableViewController.h"
#import <AlinkAppExpress/LKAppExpress.h>
#import <IMSApiClient/IMSApiClient.h>
#import <IMSLog/IMSLog.h>
#import <AlinkAppExpress/LKAppExpressEnvSet.h>
#import <IMSHUD/UIViewController+HUD.h>
#import <UITextView+Placeholder/UITextView+Placeholder.h>

#define MOBILE_CHANNEL_DEMO @"MobileChannelDemo"

#define buttonGrayColor [UIColor colorWithRed:190/255.0 green:190/255.0 blue:190/255.0 alpha:0.5/1.0]
#define buttonTitleGrayColor [UIColor colorWithRed:180/255.0 green:180/255.0 blue:180/255.0 alpha:1/1.0]
#define buttonBackgroundGrayColor  [UIColor colorWithRed:251/255.0 green:251/255.0 blue:251/255.0 alpha:1/1.0]
#define buttonGreenColor [UIColor colorWithRed:53/255.0 green:179/255.0 blue:74/255.0 alpha:1/1.0]
#define buttonBlueColor [UIColor colorWithRed:71/255.0 green:193/255.0 blue:223/255.0 alpha:1/1.0]

@interface IMSDemoConnectStateChangeListener : NSObject <LKAppExpConnectListener>
    
@property (nonatomic, weak) IMSMobileChannelTableViewController *vc;
@property (atomic, strong) NSString *connectState;
@end

@implementation IMSDemoConnectStateChangeListener

-(void)onConnectState:(LKAppExpConnectState) state {
    NSDictionary *allStatus = @{@(LKAppExpConnectStateConnected) : @"已连接",
                                @(LKAppExpConnectStateDisconnected) : @"未连接",
                                @(LKAppExpConnectStateConnecting) : @"连接中..."};

    NSString *status = allStatus[@(state)] ?: @"";;
    IMSLogDebug(MOBILE_CHANNEL_DEMO, @"连接状态变化：%@", status);
    self.connectState = status;
    if (self.vc) {
        [IMSDemoConnectStateChangeListener connectStateWithVC:self.vc status:status];
    }
}
+(void)connectStateWithVC:(IMSMobileChannelTableViewController *)vc status:(NSString *)status{
    vc.connectStateLabel.text = status;
    if ([status isEqualToString:@"已连接"]) {
        vc.connectStateBackground.backgroundColor = [UIColor colorWithRed:53/250.0 green:179/250.0 blue:74/250.0 alpha:0.8];
        vc.connectStateIcon.image = [UIImage imageNamed:@"mc_connect_icon"];
    }else if ([status isEqualToString:@"未连接"]){
        vc.connectStateBackground.backgroundColor = [UIColor lightGrayColor];
        vc.connectStateIcon.image = [UIImage imageNamed:@"mc_unconnect_icon"];
    }else{
        vc.connectStateBackground.backgroundColor = [UIColor lightGrayColor];
        vc.connectStateIcon.image = [UIImage imageNamed:@"mc_connect_icon"];
    }
}

@end

IMSDemoConnectStateChangeListener *g_connectStateListener;


@interface IMSDemoDownStreamListener : NSObject <LKAppExpDownListener>

@property (nonatomic, weak) UITextView *pushResultTextView;

- (void)onDownstream:(NSString * _Nonnull) topic data: (NSDictionary * _Nullable) data;
- (BOOL)shouldHandle:(NSString *)topic;

@end

@implementation IMSDemoDownStreamListener

- (void)onDownstream:(NSString * _Nonnull) topic data: (NSDictionary * _Nullable) data {
    NSString *result = [NSString stringWithFormat:@"topic:\n %@\ndata:\n%@", topic, data];
    self.pushResultTextView.text = result;
}

- (BOOL)shouldHandle:(NSString *)topic {
    return YES;
}

@end

typedef NS_ENUM(NSInteger, SubButtonStatus) {
    SubButtonStatusDisabled,
    SubButtonStatusSubscribable,
    SubButtonStatusUnSubscribable
};
@interface IMSMobileChannelTableViewController () <UITextFieldDelegate>
@property (weak, nonatomic) IBOutlet UITextField *publishTextField;
@property (weak, nonatomic) IBOutlet UITextView *publishParamsTextView;
@property (weak, nonatomic) IBOutlet UITextField *topicTextFiled;
@property (weak, nonatomic) IBOutlet UIButton *subOrUnsubButton;
@property (weak, nonatomic) IBOutlet UITextView *pushResultTextView;

@property (weak, nonatomic) IBOutlet UIView *subBackgroundView;
@property (weak, nonatomic) IBOutlet UISwitch *subOrUnsubAllSwitch;

@property (nonatomic, strong) IMSDemoDownStreamListener* downStreamListener;
@end

@implementation IMSMobileChannelTableViewController

+ (void)setup {
    [IMSLog registerTag:MOBILE_CHANNEL_DEMO];
    
    //[LKAppExpressEnvSet setRunEnv:LKAppExpressEnvDaily];
    
    LKAEConnectConfig *config = [[LKAEConnectConfig alloc] init];
    config.appKey = [IMSConfiguration sharedInstance].appKey;
    config.authCode = [IMSConfiguration sharedInstance].authCode;
    g_connectStateListener = [[IMSDemoConnectStateChangeListener alloc] init];
    IMSLogDebug(MOBILE_CHANNEL_DEMO, @"开始建联");
    [[LKAppExpress sharedInstance] startConnect:config connectListener:g_connectStateListener];
}

+ (UIViewController *)storyboardController {
    NSBundle *bundle = [NSBundle bundleForClass:[self class]];
    NSURL *url = [bundle URLForResource:@"IMSMobileChannelDemo" withExtension:@".bundle"];
    if (!url) {
        url = [bundle bundleURL];
    }
    NSBundle *demoBundle = [NSBundle bundleWithURL:url];
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"IMSMobileChannelDemo" bundle:demoBundle];
    NSString *name = NSStringFromClass([self class]);
    
    return [storyboard instantiateViewControllerWithIdentifier:name];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.publishTextField.delegate = self;
    self.topicTextFiled.delegate = self;
    
    self.connectStateLabel.text = g_connectStateListener.connectState;
    g_connectStateListener.vc = self;
    [IMSDemoConnectStateChangeListener connectStateWithVC:self status:g_connectStateListener.connectState];

    self.downStreamListener = [[IMSDemoDownStreamListener alloc] init];
    self.downStreamListener.pushResultTextView = self.pushResultTextView;
    [[LKAppExpress sharedInstance] addDownStreamListener:YES listener:self.downStreamListener];
    
    UITapGestureRecognizer *gestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hideKeyboard)];
    [self.tableView addGestureRecognizer:gestureRecognizer];
    
    [self changeSubOrUnsubButtonWithStatus:SubButtonStatusDisabled];
    self.publishParamsTextView.placeholder = @"请输入 JSON 格式参数";
    [self.publishParamsTextView setPlaceholderColor:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:1/1.0]];
    [self.publishTextField setValue:[UIColor colorWithRed:153/255.0 green:153/255.0 blue:153/255.0 alpha:1/1.0] forKeyPath:@"_placeholderLabel.textColor"];    
    [self.topicTextFiled setValue:[UIColor colorWithRed:153/255.0 green:153/255.0 blue:153/255.0 alpha:1/1.0] forKeyPath:@"_placeholderLabel.textColor"];
    self.tableView.backgroundColor=[UIColor colorWithRed:239.0 / 255
                                                   green:238.0 / 255
                                                    blue:241.0 / 255
                                                   alpha:1];
    self.tableView.separatorColor=[UIColor colorWithRed:234 / 255.0
                                                  green:234 / 255.0
                                                   blue:234 / 255.0
                                                  alpha:1 / 1.0];
    
    
}

-(void)changeSubOrUnsubButtonWithStatus:(SubButtonStatus)status{
    switch (status) {
        case SubButtonStatusDisabled:
            [self changeSubOrUnsubButtonWithTitle:nil borderColor: buttonGrayColor backgroundColor:buttonBackgroundGrayColor titleColor:buttonTitleGrayColor];
            self.subBackgroundView.hidden = NO;
            break;
        case SubButtonStatusSubscribable:
            [self changeSubOrUnsubButtonWithTitle:nil borderColor:buttonGreenColor backgroundColor:buttonGreenColor titleColor:[UIColor whiteColor]];
            self.subBackgroundView.hidden = YES;
            break;
        case SubButtonStatusUnSubscribable:
            [self changeSubOrUnsubButtonWithTitle:nil borderColor:buttonBlueColor backgroundColor:[UIColor whiteColor] titleColor:buttonBlueColor];
            self.subBackgroundView.hidden = YES;
            break;
        default:
            break;
    }

}
-(void)changeSubOrUnsubButtonWithTitle:(NSString *)title borderColor:(UIColor *)borderColor backgroundColor:(UIColor *)backgroundColor titleColor:(UIColor *)titleColor{
    self.subOrUnsubButton.layer.borderColor =borderColor.CGColor;
    self.subOrUnsubButton.backgroundColor = backgroundColor;
    [self.subOrUnsubButton setTitleColor:titleColor forState:UIControlStateNormal];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)onClickSetTestData:(UIBarButtonItem *)sender {
    self.publishTextField.text = @"/test/publish";
    self.publishParamsTextView.text = @"{\"input\":\"Hello World\"}";
    self.topicTextFiled.text = @"/test/subscribe";
}



- (IBAction)onClickPublish:(UIButton *)sender {
    NSString *topic = self.publishTextField.text;
    if (![topic hasPrefix:@"/"]) {
        [self ims_showHUDWithMessage:@"请输入合法的 Topic \n 合法 Topic 以 / 开头"];
        return;
    }
    
    if ([self.publishParamsTextView.text length] == 0) {
        self.publishParamsTextView.text = @"{}";
    }
    
    NSData *data = [self.publishParamsTextView.text dataUsingEncoding:NSUTF8StringEncoding];
    if (data == nil) {
        [self ims_showHUDWithMessage:@"JSON参数不合法"];
        return;
    }
    NSError *error;
    NSDictionary *params = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
    if (error) {
        NSString *msg = [NSString stringWithFormat:@"JSON参数不合法：%@", error];
        [self ims_showHUDWithMessage:msg];
        return;
    }
    
    IMSLogInfo(MOBILE_CHANNEL_DEMO, @"start publish topic: %@", topic);
    [self ims_showHUDWithIndicator];
    [[LKAppExpress sharedInstance] publish:topic params:params complete:^(NSError * _Nonnull error) {
        IMSLogInfo(MOBILE_CHANNEL_DEMO, @"publish result with error: %@", error ?: @"no error");
        dispatch_async(dispatch_get_main_queue(), ^{
            [self ims_showHUDWithMessage:error ? @"发布失败" : @"发布成功"];
        });
    }];
}

- (IBAction)onClickSubscribeWildcard:(UISwitch *)sender {
    [self ims_showHUDWithIndicator];
    if (sender.isOn) {
        [[LKAppExpress sharedInstance] subscribe:@"/#" complete:^(NSError * _Nullable error) {
            IMSLogInfo(MOBILE_CHANNEL_DEMO, @"subscribe result with error: %@", error ?: @"no error");
            
            dispatch_async(dispatch_get_main_queue(), ^{
                if (!error) {
                    self.topicTextFiled.enabled = NO;
                    self.subOrUnsubButton.enabled = NO;
                    [self.subOrUnsubButton setTitle:@"单独订阅" forState:UIControlStateNormal];
                    [self changeSubOrUnsubButtonWithStatus:SubButtonStatusDisabled];
                }else{
                    self.subOrUnsubAllSwitch.on =!self.subOrUnsubAllSwitch.on;
                }
                
                [self ims_showHUDWithMessage:error ? @"订阅全部失败" : @"订阅全部成功"];
            });
        }];
    } else {
        [[LKAppExpress sharedInstance] unsubscribe:@"/#" complete:^(NSError * _Nullable error) {
            IMSLogInfo(MOBILE_CHANNEL_DEMO, @"unsubscribe result with error: %@", error ?: @"no error");
            dispatch_async(dispatch_get_main_queue(), ^{
                [self ims_showHUDWithMessage:error ? @"取消订阅全部失败" : @"取消订阅全部成功"];
                
                if (!error) {
                    self.topicTextFiled.enabled = YES;
                    self.subOrUnsubButton.enabled = YES;
                    [self.subOrUnsubButton setTitle:@"单独订阅" forState:UIControlStateNormal];
                    [self changeSubOrUnsubButtonWithStatus:SubButtonStatusSubscribable];
                }else{
                    self.subOrUnsubAllSwitch.on =!self.subOrUnsubAllSwitch.on;
                }
            });
        }];
    }
}

- (IBAction)onClickSubscribeOrUnsubsribe:(UIButton *)sender {
    NSString *topic = self.topicTextFiled.text;
    if (![topic hasPrefix:@"/"]) {
        [self ims_showHUDWithMessage:@"请输入合法的 Topic，合法 Topic 以 / 开头"];
        return;
    }
    
    [self ims_showHUDWithIndicator];
    if ([[sender titleForState:UIControlStateNormal] isEqualToString:@"单独订阅"]) {
        IMSLogInfo(MOBILE_CHANNEL_DEMO, @"start subscribe topic: %@", topic);
        [[LKAppExpress sharedInstance] subscribe:topic complete:^(NSError * _Nullable error) {
            IMSLogInfo(MOBILE_CHANNEL_DEMO, @"subscribe topic: %@, result with error: %@", topic, error ?: @"no error");
            dispatch_async(dispatch_get_main_queue(), ^{
                if (!error) {
                    [sender setTitle:@"取消订阅" forState:UIControlStateNormal];
                    [self changeSubOrUnsubButtonWithStatus:SubButtonStatusUnSubscribable];
                }
                [self ims_showHUDWithMessage:error ? @"订阅失败" : @"订阅成功"];
            });
        }];
    } else {
        IMSLogInfo(MOBILE_CHANNEL_DEMO, @"start unsubscribe topic: %@", topic);
        [[LKAppExpress sharedInstance] unsubscribe:topic complete:^(NSError * _Nullable error) {
            IMSLogInfo(MOBILE_CHANNEL_DEMO, @"unsubscribe topic: %@, result with error: %@", topic, error ?: @"no error");
            dispatch_async(dispatch_get_main_queue(), ^{
                if (!error) {
                    [sender setTitle:@"单独订阅" forState:UIControlStateNormal];
                    [self changeSubOrUnsubButtonWithStatus:SubButtonStatusSubscribable];
                }
                [self ims_showHUDWithMessage:error ? @"取消订阅失败" : @"取消订阅成功"];
            });
        }];
    }
}

    
- (IBAction)onClickClearPushResult:(UIButton *)sender {
    self.pushResultTextView.text = @"";
}
    
/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

-(CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section{
    return 0.1;
}
-(CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    if (section == 0) {
        return 0.1;
    }else{
        return 30.0;
    }
}



- (void)tableView:(UITableView *)tableView
willDisplayHeaderView:(UIView *)view
       forSection:(NSInteger)section {
    // Text Color
    UITableViewHeaderFooterView *header = (UITableViewHeaderFooterView *)view;
    [header.textLabel setTextColor:[UIColor colorWithRed:155 / 255.0
                                                   green:155 / 255.0
                                                    blue:155 / 255.0
                                                   alpha:1 / 1.0]];
    header.textLabel.font = [UIFont fontWithName:@"PingFangSC-Regular" size:12];
}



#pragma mark - delegate
    
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
    return YES;
}
    
- (BOOL)textFieldShouldEndEditing:(UITextField *)textField {
    return YES;
}
    
- (BOOL)textFieldShouldClear:(UITextField *)textField {
    return YES;
}
    
- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

- (void)hideKeyboard {
    [self.view endEditing:YES];
}

@end
